--[[
	JPack
]]


--[[===================================
			local
=====================================]]

local JPack={
	packingBank=false,
	bankOpened=false,
	guildbankOpened=false,
	deposit=false,
	draw=false,
	bagGroups={},
	packingGroupIndex=1,
	packingBags={},
	debug=false,
}

local version = "0.5.9.1"	--GetAddOnMetadata('JPack', 'Version')
local L = JPackLocale

local nextTryTime=0
local interval=0.1
local bagSize=0
local packingBags={}
local JPACK_MAXMOVE_ONCE=3
local current,to,lockedSlots

-- 整理状态
local JPACK_STEP=0
local JPACK_STARTED=1
local JPACK_DEPOSITING=2
local JPACK_DRAWING=3
local JPACK_STACKING=4
local JPACK_STACK_OVER=5
local JPACK_SORTING=6
local JPACK_PACKING=7
local JPACK_START_PACK=8
local JPACK_STOPPED=0


--[[===================================
			lib
=====================================]]

if table.getn == nil then
	local fun, err = loadstring("return function(tb) return #tb end")
	table.getn = fun()
end 

local function print(msg,r,g,b)
	if not msg then return end
	if msg == '' then return end
	if msg == ' ' then return end
	
	r, g, b = r or .36, g or .74, b or .74
	DEFAULT_CHAT_FRAME:AddMessage('JPack: '..msg,r,g,b)
end

local function debug(msg)
	if(JPack.debug)then
		print(msg,1,0,0)
	end
end

local function IndexOfTable(t,v)
	for i=1,table.getn(t) do
		if(v==t[i])then
			return i
		end
	end
	return 0
end

--[[
获得一个 JPack 格式的物品对象
]]
local function getJPackItem(bag,slot)
	local link = GetContainerItemLink(bag,slot) 
	if(link == nil)then return nil end
	local item={}
	item.slotId=c
	item.name, item.link, item.rarity, 
	item.level, item.minLevel, item.type, item.subType, item.stackCount,
	item.equipLoc, item.texture = GetItemInfo(link)
	item.itemid = tonumber(link:match("item:(%d+):"))
	return item
end

--背包是否准备好了（无锁定物品）
local function isBagReady(bag)
	for i=1,GetContainerNumSlots(bag) do
		local _, _, locked = GetContainerItemInfo(bag,i)
		if(locked)then return false end
	end
	return true
end

--是否所有背包准备好了
local function isAllBagReady()
	for i=1,table.getn(JPack.bagGroups) do
		for j=1,table.getn(JPack.bagGroups[i]) do
			if(not isBagReady(JPack.bagGroups[i][j])) then return false end
		end
	end
	return true
end

--[[
return 1xxx 非垃圾 ，0xxx for 垃圾，xxx为（999-JPACK_ORDER中所定的位置）
]]--
local function getPerffix(item)
	if(item==nil)then return nil end
	
	--按名称获取顺序
	local i=IndexOfTable(JPACK_ORDER,item.name)
	if(i<=0)then
		--按子类别获取顺序
		i=IndexOfTable(JPACK_ORDER,"#"..item.type.."##"..item.subType)
	else
		--名称匹配直接返回
		return '1'..string.format("%3d",999-i)
	end
	if(i<=0)then
		--按子类别获取顺序
		i=IndexOfTable(JPACK_ORDER,"##"..item.subType)
	end
	if(i<=0)then
		--按类别获取顺序
		i=IndexOfTable(JPACK_ORDER,"#"..item.type)
	end
	if(i<=0)then
		--默认类别顺序
		i=IndexOfTable(JPACK_ORDER,"#")
	end
	if(i<=0)then
		--默认顺序
		i=999
	end
	local s=string.format("%3d",999-i)
	--灰色物品、可装备的非优秀物品
	if(item.rarity==0)then
		return "00"..s
	elseif(IsEquippableItem(item.name) and item.type~=JPackLocale.TYPE_BAG and item.subType~=JPackLocale.TYPE_FISHWEAPON) then 
		if(item.rarity <= 1 ) then
			return '01'..s
		end
	end
	return "1"..s
	
end

--[[
bagIds = {1,3,5}
packIndex ---JPack的index
bagID --- wow 的bagId
slotId ---- wow 的slotId
]]--

--[[
bag  背包
slot  位置
return 是否需要保存到银行
]]--
local function shouldSaveToBank(bag,slot)
	local item=getJPackItem(bag,slot)
	return item~=nil and ((IndexOfTable(JPACK_DEPOSIT,"#"..item.type)>0) or (IndexOfTable(JPACK_DEPOSIT,"##"..item.subType)>0) or (IndexOfTable(JPACK_DEPOSIT,item.name)>0))
end

--[[
bag  背包
slot  位置
return 是否需要从银行取出
]]--
local function shouldLoadFromBank(bag,slot)
	local item=getJPackItem(bag,slot)
	return item~=nil and ((IndexOfTable(JPACK_DRAW,"#"..item.type)>0) or (IndexOfTable(JPACK_DRAW,"##"..item.subType)>0) or (IndexOfTable(JPACK_DRAW,item.name)>0))
end

--[[
bags  一组背包
bag    当前背包在bags中位置
slot   当前背包slot
return  前一个背包位置的  bag,slot
]]--
local function getPrevSlot(bags,bag,slot)
	if(slot>1)then
		slot=slot-1
	elseif(bag>1)then
		bag=bag-1
		slot=GetContainerNumSlots(bags[bag])
	else
		bag=-1
	end
	return bag,slot
end

--[[
比较用的字符串,与排序直接相关的函数
]]--
local function getCompareStr(item)
	if(not item)then
		return nil
	elseif(not item.compareStr)then
		local _,_,textureType,textureIndex=string.find(item.texture,"\\.*\\([^_]+_?[^_]*_?[^_]*)_?(%d*)")
		if(not item.rarity)then item.rarity='1' end
		item.compareStr= getPerffix(item).." "..item.rarity..item.type.." "..item.subType.." "..textureType.." "
			..string.format("%2d",item.minLevel).." "..string.format("%2d",item.level).." "..(textureIndex or "00")..item.name
	end
	return item.compareStr
end

--[[
比较两个物品
return 1 a 在前
return -1 b 在前
]]--
local function compare(a, b)
	local ret=0
	if(a==b)then
		ret= 0
	elseif(a==nil)then
		ret= -1
	elseif(b==nil)then
		ret= 1
	elseif(a.name==b.name)then
		ret= 0
	else
		local sa = getCompareStr(a)
		local sb = getCompareStr(b)
		if(sa>sb)then
			ret= 1
		elseif(sa<sb) then
			ret= -1
		end
	end
	return ret
end

local function swap(items,i,j)
	local y=items[i]
	items[i]=items[j]
	items[j]=y
end

local function qsort(items,from,to)

	local i,j=from,to
	local ix=items[i]
	local x=i
	while(i<j) do
		while(j>x) do
			if(compare(items[j], ix)==1)then
				swap(items,j,x)
				x=j
      		else
   				j=j-1
			end
  		end
		while(i<x)do
			if(compare(items[i], ix)==-1)then
				swap(items,i,x)
				x=i
			else
   				i=i+1
			end
  		end
 	end
	if(x-1 > from) then
		qsort(items,from,x-1)
	end
	if(x+1 < to) then
		qsort(items,x+1,to)
	end
end

--[[
排序 items ,并返回一个新的排序过的 items,原 items 不变
]]
local function jsort(items)
	local clone = {}--Item:new[items.length]
	for i=1,bagSize do
		clone[i] = items[i]
	end
	qsort(clone,1,bagSize)
	return clone
end

--local moving=false
local function sortTo(_current, _to)
	current=_current
	to=_to
	lockedSlots={}
	JPACK_STEP=JPACK_PACKING
end

--[[===================================
		Main
=====================================]]

--保存到银行
local function saveToBank()
	if(not JPack.bankOpened)then return end
	
	--保存
	for k,v in pairs(JPack.bankSlotTypes) do
		--针对每种不同的背包,k is type,v is slots
		local bkTypes,bagTypes=JPack.bankSlotTypes[k],JPack.bagSlotTypes[k]
		local bkBag,bag=table.getn(bkTypes),table.getn(bagTypes)
		local bkSlot,slot=GetContainerNumSlots(bkTypes[bkBag]),GetContainerNumSlots(bagTypes[bag])
		--保存
		while(true) do
			while(bkBag>0 and GetContainerItemLink(bkTypes[bkBag],bkSlot))do
				--直到找到一个空格
				bkBag,bkSlot=getPrevSlot(bkTypes,bkBag,bkSlot)
			end
			
			while(bag>0 and (not shouldSaveToBank(bagTypes[bag],slot)))do
				bag,slot=getPrevSlot(bagTypes,bag,slot)
			end
			
			if(bkBag<=0 or bag <=0 or bkSlot<=0 or slot<=0)then 
				debug("break to save")
				break
			end
			if (CursorHasItem() or CursorHasMoney() or CursorHasSpell()) then
				print(L["WARN"],1,0,0)
			end
			PickupContainerItem(bagTypes[bag],slot)
			PickupContainerItem(bkTypes[bkBag],bkSlot)
			--next
			bkBag,bkSlot=getPrevSlot(bkTypes,bkBag,bkSlot)
			bag,slot=getPrevSlot(bagTypes,bag,slot)
		end
	end
end

--从银行取出
local function loadFromBank()
	if(not JPack.bankOpened)then return end
	
	--保存
	for k,v in pairs(JPack.bankSlotTypes) do
		--针对每种不同的背包,k is type,v is slots
		local bkTypes,bagTypes=JPack.bankSlotTypes[k],JPack.bagSlotTypes[k]
		local bkBag,bag=table.getn(bkTypes),table.getn(bagTypes)
		local bkSlot,slot=GetContainerNumSlots(bkTypes[bkBag]),GetContainerNumSlots(bagTypes[bag])
		--保存
		while(true) do
			while(bag>0 and GetContainerItemLink(bagTypes[bag],slot))do
				--直到找到一个空格
				bag,slot=getPrevSlot(bagTypes,bag,slot)
			end
			while(bkBag>0 and (not shouldLoadFromBank(bkTypes[bkBag],bkSlot)))do
				bkBag,bkSlot=getPrevSlot(bkTypes,bkBag,bkSlot)
			end
			
			if(bkBag<=0 or bag <=0 or bkSlot<=0 or slot<=0)then 
				break
			end
			if (CursorHasItem() or CursorHasMoney() or CursorHasSpell()) then
				print(L["WARN"],1,0,0)
			end
			PickupContainerItem(bkTypes[bkBag],bkSlot)
			PickupContainerItem(bagTypes[bag],slot)
			--next
			bkBag,bkSlot=getPrevSlot(bkTypes,bkBag,bkSlot)
			bag,slot=getPrevSlot(bagTypes,bag,slot)
		end
	end
end


--[[
将背包按类型分组
bagGroups
bankGroups
bagTypes
packingTypeIndex
packingBags
]]--
local function groupBags()
	local bagTypes={}
	bagTypes[JPackLocale.TYPE_BAG]={}
	bagTypes[JPackLocale.TYPE_BAG][1]=0
	for i=1,4 do
		local name=GetBagName(i)
		if(name)then
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, subType, itemStackCount,
itemEquipLoc, itemTexture = GetItemInfo(name)
			debug("Bag["..i.."]Type："..subType)
			if(bagTypes[subType]==nil)then
				bagTypes[subType]={}
			end
			local t = bagTypes[subType]
			t[table.getn(t)+1]=i
		end
	end

	local bankSlotTypes={}
	if(JPack.bankOpened)then
		bankSlotTypes[JPackLocale.TYPE_BAG]={}
		bankSlotTypes[JPackLocale.TYPE_BAG][1]=-1
		for i=5,11 do
			local name=GetBagName(i)
			if(name)then
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, subType, itemStackCount,
itemEquipLoc, itemTexture = GetItemInfo(name)
				if(bankSlotTypes[subType]==nil)then
					bankSlotTypes[subType]={}
				end
				local t = bankSlotTypes[subType]
				t[table.getn(t)+1]=i
			end
		end
	end
	JPack.bagSlotTypes=bagTypes
	JPack.bankSlotTypes=bankSlotTypes
	local j=1
	for k,v in pairs(bankSlotTypes) do
		JPack.bagGroups[j]=v
		j=j+1
	end
	for k,v in pairs(bagTypes) do
		JPack.bagGroups[j]=v
		j=j+1
	end
	
end

--[[
获取PackingBags的 JPack格式物品table
]]--
local function getPackingItems()
	local c=1
	local items={}
	if JPackDB.asc then
		for i=1,table.getn(JPack.packingBags) do
			local num = GetContainerNumSlots(JPack.packingBags[i]) 
			for j = 1,num do
				items[c]=getJPackItem(JPack.packingBags[i],j)
				c = c+1
			end
		end
	else
		for i=table.getn(JPack.packingBags),1,-1 do
			local num = GetContainerNumSlots(JPack.packingBags[i]) 
			for j = num,1,-1 do
				items[c]=getJPackItem(JPack.packingBags[i],j)
				c = c+1
			end
		end
	end
	return items,c-1
end

--[[
开始对packingBags 进行整理
]]--
local function startPack()
	local items,count = getPackingItems()
	bagSize=count
	local sorted = jsort(items)
	debug("sorted...")
	--[[for i=1,table.getn(sorted) do
		debug(getCompareStr(sorted[i]))
	end]]
	sortTo(items,sorted)
end

--[[
将 PackIndex 转换为 BagId,SlotId
]]--
local function getSlotId(packIndex)
	local slot=packIndex
	if JPackDB.asc then
		for i=1,table.getn(JPack.packingBags) do
			local num=GetContainerNumSlots(JPack.packingBags[i]) 
			if(slot<=num)then
				return JPack.packingBags[i],slot
			end
			slot = slot - num
		end
	else
		for i=table.getn(JPack.packingBags),1,-1 do
			local num=GetContainerNumSlots(JPack.packingBags[i]) 
			if(slot<=num)then
				return JPack.packingBags[i],1+num-slot
			end
			slot = slot - num
		end
	end
	return -1,-1
end

--[[
oldIndex  原JPack物品位置索引
newIndex 新JPack物品位置索引
把物品从oldIndex 移动到 newIndex
]]
local function moveTo(oldIndex,newIndex)
	PickupContainerItem(getSlotId(oldIndex))
	PickupContainerItem(getSlotId(newIndex))
end

--[[
某位置是否被锁定,与blizzard函数不同的是，空格也可能被锁定
]]--
local function isLocked(index)
	local il=IndexOfTable(lockedSlots,index)
	local texture, itemCount, locked, quality, readable = GetContainerItemInfo(getSlotId(index))
	if(texture==nil)then
		locked= il >0
	elseif(il>0)then
		table.remove(lockedSlots,il)
	end
	return locked
end

--[[
items 物品数组
key  待查找物品名字
return 最后一个可以移动的此物品位置
]]--
local function GetLastItemIndex(items,key)
	local i=bagSize
	while(i>0)do
		if(items[i] ~= nil and not isLocked(i) and items[i].name == key)then
			return i
		end
		i= i-1
	end
	return -1
end

--[[
移动一次, 返回是否继续
]]--
local function moveOnce()
	local continue=false
	local i=1
	local lockCount=0
	while(to[i] ~=nil)do
		local locked=isLocked(i)
		if(locked==nil)then locked=false end
		if(locked)then
			lockCount=lockCount+1
		end
		if(lockCount>JPACK_MAXMOVE_ONCE)then
			return true
		end
		if(current[i] == nil or to[i].name ~= current[i].name)then
			continue = true
			if(not locked)then
				local slot =GetLastItemIndex(current, to[i].name)
				if(slot ~= -1)then
					moveTo(slot,i) -- 移动物品
					local x=current[slot]
					current[slot]=current[i]
					current[i]=x
					if(current[slot]==nil)then
						--锁定空格
						lockedSlots[table.getn(lockedSlots)+1]=i
					end
				end
			end
			
		end
		i=i+1
	end
	return continue or lockCount>0
end

--堆叠一次，返回是否结束
local function stackOnce()
	local bags,bag,item,slotInfo
	if(JPack.bankOpened)then
		bags={11,10,9,8,7,6,5,-1,4,3,2,1,0}
	else
		bags={4,3,2,1,0}
	end
	local pendingStack={}
	local complet=true
	for i=1,#bags do
		bag = bags[i]
		for slot = GetContainerNumSlots(bag),1,-1 do
			local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot)
			item = getJPackItem(bag,slot)
			if(item)then
				if(not locked)then
					if(itemCount < item.stackCount)then
						-- item.name ==> item.itemid
						slotInfo = pendingStack[item.itemid]
						if(slotInfo)then
							PickupContainerItem(bag,slot)
							PickupContainerItem(slotInfo[1],slotInfo[2])
							pendingStack[item.itemid]=nil
							complet = false
						else
							pendingStack[item.itemid]={bag,slot}
						end
					end
				else
					complet = false
				end
			end
		end
	end
	return complet
end



--[[
		=== GuildBank ===
		
	http://wowprogramming.com/docs/api/
			GetCurrentGuildBankTab
			PickupGuildBankItem
			GetCurrentGuildBankTab
			GetGuildBankItemInfo
			GetGuildBankItemLink
			GetGuildBankTabInfo
			GetGuildBankTabPermissions
			GetNumGuildBankTabs
]]


--[[===================================
		Others
=====================================]]

JPackFrame=CreateFrame("Frame",nil,UIParent)

PackOnBagOpen=true
function JPackFrame:OnEvent()
	debug(event)
	if(event=="VARIABLES_LOADED")then
		self:Init()
	elseif(event=="BAG_OPEN")then
		if(PackOnBagOpen)then
			pack()
		end
	elseif(event=="BANKFRAME_OPENED")then
		JPack.bankOpened=true
		debug("bank opened")
	elseif(event=="BANKFRAME_CLOSED")then
		JPack.bankOpened=false
		debug("bank closed")
	elseif(event=="GUILDBANKFRAME_OPENED")then
		JPack.guildbankOpened=true
		debug("guildbank opened")
	elseif(event=="GUILDBANKFRAME_CLOSED")then
		JPack.guildbankOpened=false
		debug("guildbank closed")
	end
end

--[[
每帧执行，判断当前整理进度并执行相应操作
]]--
function JPackFrame:OnUpdate()
	if(GetTime() < nextTryTime)then return end
	nextTryTime = GetTime() + interval
	if(JPACK_STEP==JPACK_STARTED)then
		if(stackOnce())then
			JPACK_STEP=JPACK_STACK_OVER
		end
	elseif(JPACK_STEP==JPACK_STACK_OVER)then
		debug("JPACK_STEP==JPACK_STACK_OVER, 开始向银行保存")
		if(isAllBagReady())then
			debug("堆叠完毕,JPack_STEP=JPACK_STACK_OVER")
			if(JPack.deposit)then
				debug("saveToBank()")
				saveToBank()
			end
			JPACK_STEP=JPACK_DEPOSITING
		end
	elseif(JPACK_STEP==JPACK_DEPOSITING)then
		debug("JPACK_STEP==JPACK_DEPOSITING, 开始从银行提取")
		if(isAllBagReady())then
			debug("保存物品完毕,JPack_STEP=JPACK_DEPOSITING")
			if(JPack.draw)then
				debug("loadFromBank()")
				loadFromBank()
			end
			JPACK_STEP=JPACK_START_PACK
		end
	elseif(JPACK_STEP==JPACK_START_PACK)then
		debug("开始整理,JPACK_STEP="..JPACK_STEP)
		if(isAllBagReady)then
			JPack.packingGroupIndex=1
			JPack.packingBags=JPack.bagGroups[1]
			--计算排序
			startPack()
			JPACK_STEP=JPACK_PACKING
		end
	elseif(JPACK_STEP==JPACK_PACKING)then
		--排序结束
		--移动物品
		local continueMove=moveOnce()
		if(not continueMove)then
			JPack.packingGroupIndex=JPack.packingGroupIndex + 1
			debug("index"..JPack.packingGroupIndex)
			JPack.packingBags=JPack.bagGroups[JPack.packingGroupIndex]
			debug("JPack.bagGroups . size = "..table.getn(JPack.bagGroups))
			for i=1,table.getn(JPack.bagGroups) do
				for j=1,table.getn(JPack.bagGroups[i]) do
					debug("i"..i.."j"..j..":"..JPack.bagGroups[i][j])
				end
			end
			if(JPack.packingBags==nil)then
				--整理结束
				JPACK_STEP=JPACK_STOPPED
				JPack.bagGroups={}
				print(L["COMPLETE"])
				JPackFrame:SetScript("OnUpdate",nil)
				current=nil
				to=nil
			else
				debug("Packing "..JPack.packingGroupIndex)
				startPack()
			end
		end
	end
end

JPackFrame:RegisterEvent("VARIABLES_LOADED")

JPackFrame:SetScript("OnEvent",JPackFrame.OnEvent)

function JPackFrame:Init()
	if not JPackDB then JPackDB = {} end
	if not JPackMMIconDB then JPackMMIconDB = {} end
	if JPack_LoadMMIcon then JPack_LoadMMIcon() end
	print(string.format('%s %s', version, L["HELP"]))
	JPackFrame:RegisterEvent("BAG_OPEN")
	JPackFrame:RegisterEvent("BANKFRAME_OPENED")
	JPackFrame:RegisterEvent("BANKFRAME_CLOSED")
	JPackFrame:RegisterEvent("GUILDBANKFRAME_CLOSED")
	JPackFrame:RegisterEvent("GUILDBANKFRAME_OPENED")
	SlashCmdList["JPACK"] = JPack_Slash
	SLASH_JPACK1 = "/jpack"
	SLASH_JPACK2 = "/jp"

end

local function pack()
	if (CursorHasItem() or CursorHasMoney() or CursorHasSpell()) then
		print(L["WARN"],2,0.28,2)
	else
		groupBags()
		JPACK_STEP=JPACK_STARTED
		JPackFrame:SetScript("OnUpdate",JPackFrame.OnUpdate)
	end
end

function JPack_Slash(msg)
	local a,b,c=strfind(msg, "(%S+)")
	c = c and string.lower(c) or nil
	if(c=="minimap" or c=="mm")then
		if JPack_ToggleMMIcon then JPack_ToggleMMIcon() end
	elseif(c=="asc")then
		JPack_Pack(nil, 1)
	elseif(c=="desc")then
		JPack_Pack(nil, 2)
	elseif(c=="deposit" or c=="save")then
		JPack_Pack(1)
	elseif(c=="draw" or c=="load")then
		JPack_Pack(2)
	elseif(c=="debug")then
		JPack.debug=true
	elseif(c=="nodebug")then
		JPack.debug=false
	elseif(c=="help")then
		local text = "%s - |cffffffff%s|r"
		print(L["Slash command"]..": /jpack |cffffffffor|r /jp")
		print(string.format(text, "/jp", L["Pack"]))
		print(string.format(text, "/jp minimap |cffffffffor|r mm", L["Toggle minimap icon"]))
		print(string.format(text, "/jp asc", L["Set sequence to asc"]))
		print(string.format(text, "/jp desc", L["Set sequence to desc"]))
		print(string.format(text, "/jp deposit |cffffffffor|r save", L["Save to the bank"]))
		print(string.format(text, "/jp draw |cffffffffor|r load", L["Load from the bank"]))
		print(string.format(text, "/jp debug", L["Show debug info, to check pack or item category info"]))
		print(string.format(text, "/jp nodebug", L["Close debug info"]))
		print(string.format(text, "/jp help", L["Print help info"]))
	elseif not c then
		JPack_Pack()
	elseif c then
		local c = string.format(": |cffff0000%s|r , ", c)
		print(L["Unknown command"]..c..L["HELP"])
	end
end


--[[===================================
			API
=====================================]]

--[[
	JPack_Pack(access, order)
	access	
			1	save
			2	load
			nil	no action
	order		
			1	asc
			2	desc
			nil	last-time-order or default
]]
function JPack_Pack(access, order)
	JPack.deposit=false
	JPack.draw=false
	if access then
		if access == 1 then
			JPack.deposit=true
			JPack.draw=false
		elseif access == 2 then
			JPack.deposit=false
			JPack.draw=true
		end
	end
	if order == 1 then
		JPackDB.asc=true
	elseif order == 2 then
		JPackDB.asc=false
	end
	
	pack()
end

function JPack_OnClick(_, button)
	--button = button or GetMouseButtonClicked()
	if ( not InCombatLockdown() ) then
		local access, order
		if ( button == "LeftButton" ) then
			if IsShiftKeyDown() then
				access = 1
			elseif IsControlKeyDown() then
				access = 2
			end
		elseif ( button == "RightButton" ) then
			if IsShiftKeyDown() then
				order = 1
			elseif IsControlKeyDown() then
				order = 2
			end
		end
		JPack_Pack(access, order)
	end
end


--[[
	JPack_CreateButton(point, region, relativePoint, offsetX, offsetY)
	Similar to "SetPoint": http://wowprogramming.com/docs/widgets/Region/SetPoint
]]
function JPack_CreateButton(point, region, relativePoint, offsetX, offsetY)
	local button = CreateFrame("Button", nil, region, "UIPanelButtonTemplate")
	button:SetPoint(point, region, relativePoint, offsetX, offsetY)
	--button:SetScript("OnClick", JPack_OnClick)
	button:SetScript("OnMouseUp", JPack_OnClick)
	button:SetWidth(50)
	button:SetHeight(20)
	button:SetText("JPack")
	
	return button
end
